'omaLib SDL wrapper library 
' Version 2.0
'By SJ Zero
'
'Changlog
'1.0	2005-05-22	SJ Zero		Initial Release
'2.0	2014-11-13	SJ Zero		Converted to FreeBASIC 1.0.0


#include once "SDL/SDL_ttf.bi"
#include once "SDL/SDL.bi" 

'****WRAPPER FUNCTION DECLARATIONS 
DECLARE SUB omaBlitImage(x AS INTEGER,y AS INTEGER,image AS sdl_surface PTR)
DECLARE SUB omaqfakBlitImage(x AS INTEGER,y AS INTEGER,number AS INTEGER, image AS sdl_surface PTR)  
DECLARE SUB omaPageFlip() 
DECLARE SUB omaSDL_Init() 
declare sub omaKillSurface(Picture AS SDL_Surface PTR)
DECLARE FUNCTION omaMultiKey (x as integer) as integer
DECLARE FUNCTION omagetSingleEvent AS INTEGER 
DECLARE SUB omaDebugEvent (Text as string) 
DECLARE SUB omaDebugEventsActive (status AS INTEGER) 
DECLARE FUNCTION omaLoadMaskedGraphic(filename AS STRING) AS sdl_Surface PTR 
DECLARE SUB omaQuit () 
DECLARE SUB omaGetScreenDimensions (BYREF w AS INTEGER,BYREF h AS INTEGER) 
DECLARE SUB omaSetFramePeriod (x AS INTEGER)
declare sub omaPaletteOverlay
DECLARE SUB omaPrint(BYVAL x AS INTEGER, BYVAL y AS INTEGER, BYVAL clr AS SDL_Color, text AS STRING)

DIM SHARED omaFont AS TTF_Font Ptr


'******Pixel surface declarations 
DIM SHARED omaScreenSurface AS SDL_Surface PTR 

DIM SHARED omaFramesPerSecond as integer
DIM SHARED omaFramePeriod  as integer
DIM SHARED omaDebugEvents  as integer
DIM SHARED omaSDL_Started as integer
dim shared omaMouseX as integer
dim shared omaMouseY as integer
dim shared omaMouseButtons as integer
dim shared omaMeasuredFramePeriod as long
dim shared omaMeasuredFrameRate as Double

CONST omaFONTSIZE	 = 12			'' in points


SUB omaQuit () 
    TTF_Quit()
    SDL_Quit() 
    END 
END SUB 

SUB omaSetFramesPerSecond (x AS INTEGER)
    omaFramePeriod = (1! / x) * 1000
    
END SUB

FUNCTION omaLoadMaskedGraphic(filename AS STRING) AS sdl_Surface PTR 
    print "About to load file ";filename
    DIM LoadedFile AS sdl_surface PTR 
    DIM ConvertedFile AS sdl_surface PTR 

    
    LoadedFile = SDL_LoadBMP(filename) 
    print "Loaded file"
    
    IF LoadedFile = 0 THEN 
	PRINT Filename + ".bmp failed to load!" 
	omaQuit() 
END IF 
    print "about to convert file"
    ConvertedFile = SDL_DisplayFormat(LoadedFile) 
    print "Freeing original file"
    SDL_FreeSurface LoadedFile 
    print "Setting colour key to hot pink"
    SDL_SetColorKey ConvertedFile, SDL_SrcColorKey, SDL_MapRGB(ConvertedFile->Format, 255, 0, 255)
    print "Returning ptr"
    omaLoadMaskedGraphic = ConvertedFile 
END FUNCTION 
SUB omaKillSurface(Picture AS SDL_Surface PTR)
    SDL_FreeSurface Picture 
END SUB


SUB omaPageFlip() 
    IF omaSDL_Started = 0 THEN EXIT SUB
    
       
    STATIC omaOldTimer as integer
    static omaOlderTimer as integer

    SDL_Flip omaScreenSurface 
    
    SDL_PumpEvents 
    omaMouseButtons = SDL_GetMouseState(@omaMouseX, @omaMouseY)

    omaMeasuredFramePeriod = SDL_GetTicks - (omaOlderTimer)
    
    omaMeasuredFrameRate = 1! / (omaMeasuredFramePeriod)
   dim x as integer
    x = omaOldTimer - sdl_GetTicks
    omaOlderTimer = SDL_GetTicks
	if x > 0 then SDL_Delay (x)
    omaOldTimer = sdl_GetTicks () + omaFramePeriod 

END SUB 
sub omaCullBlitImage (x1 as integer, x2 as integer, w as integer, h as integer, x as integer, y as integer, image as SDL_Surface ptr)
    DIM Rectangle AS SDL_Rect 
    
    DIM Rectangle2 AS SDL_Rect 
    
    Rectangle.x = 0 
    Rectangle.y = 0 
    rectangle.w = image->w 
    rectangle.h = image->h 
    Rectangle2.x = x 
    Rectangle2.y = y 
    IF Rectangle2.x > omaScreenSurface->w OR rectangle2.y > omaScreenSurface->h THEN 
    ELSE
	SDL_BlitSurface image, @rectangle, omaScreenSurface, @rectangle2 
    END IF 
    SDL_SetClipRect omaScreenSurface, 0
    
end sub

SUB omaBlitImage(x AS INTEGER,y AS INTEGER,image AS sdl_surface PTR) 
    DIM Rectangle AS SDL_Rect 
    
    DIM Rectangle2 AS SDL_Rect 
    
    
    Rectangle.x = 0 
    Rectangle.y = 0 
    rectangle.w = image->w 
    rectangle.h = image->h 
    Rectangle2.x = x 
    Rectangle2.y = y 
    IF Rectangle2.x > omaScreenSurface->w OR rectangle2.y > omaScreenSurface->h THEN 
    ELSE
	SDL_BlitSurface image, @rectangle, omaScreenSurface, @rectangle2 
    END IF      
END SUB 

SUB omaSDL_Init () 
    
    omaSDL_Started = 1
    
    DIM x               AS STRING 
    DIM Xres            AS INTEGER 
    DIM Yres            AS INTEGER 
    DIM depth           AS INTEGER 
    DIM ScreenOptions   AS INTEGER 
    DIM fileno          AS INTEGER 
    
    'Before we open the screen, we have the simplest possible config.cfg reader. 
    'Later versions will use GetParameter(x,y) to make config options on one line. 
    fileno = FREEFILE 
    OPEN "config.cfg" FOR INPUT AS #fileno 
    WHILE NOT EOF(fileno) 
	INPUT #fileno, x 
	x = LCASE(LTRIM(RTRIM(x))) 
	
	SELECT CASE x 
	CASE "xres" 
	    INPUT #fileno, x 
	    XRES = VAL(LCASE(LTRIM(RTRIM(x)))) 
	CASE "yres" 
	    INPUT #fileno, x 
	    YRES = VAL(LCASE(LTRIM(RTRIM(x)))) 
	CASE "depth" 
	    INPUT #fileno, x 
	    depth = VAL(LCASE(LTRIM(RTRIM(x)))) 
	CASE "sdl_hwsurface" 
	    screenoptions = screenoptions OR SDL_HWSURFACE 
	CASE "sdl_swsurface" 
	    screenoptions = screenoptions OR SDL_SWSURFACE      
	CASE "sdl_doublebuf" 
	    screenoptions = screenoptions OR SDL_DOUBLEBUF 
	CASE "sdl_fullscreen" 
	    screenoptions = screenoptions OR SDL_FULLSCREEN 
	CASE "sdl_hwaccel" 
	    screenoptions = screenoptions OR SDL_HWACCEL 
	CASE "sdl_hwpalette" 
	    screenoptions = screenoptions OR SDL_HWPALETTE 
	END SELECT 
	
	'SDL_DOUBLEBUF )'or SDL_FULLSCREEN 
    WEND 
    CLOSE #fileno 
    IF XRES = 0 AND YRES = 0 THEN 
	omaDebugEvent "There was no configuration file! Using default resolution!" 
	XRES = 640 
	YRES = 480 
	Depth = 32
	ScreenOptions = SDL_SWSURFACE OR SDL_RLEACCEL OR SDL_RLEACCELOK OR SDL_ASYNCBLIT 
    END IF 
    
    SDL_Init (SDL_INIT_VIDEO) 
    ScreenOptions = ScreenOptions OR SDL_SWSURFACE OR SDL_RLEACCEL OR SDL_RLEACCELOK OR SDL_ASYNCBLIT 
    
    omaScreenSurface = SDL_SetVideoMode(XRES, YRES, depth, screenoptions) 
    
    
    IF omaScreenSurface = 0 THEN 
	omaDebugEvent "There was an error initilizing with SDL!" 
	omaDebugEvent "Parameters" 
	omaDebugEvent "Resolution: "+STR(XRES)+" x "+STR(YRES) 
	omaDebugEvent "Depth:"+STR(depth) 
	omaDebugEvent "Screen Options: "+STR(screenoptions)+" <- this is supposed to be a number" 
	omaDebugEvent "" 
	omaDebugEvent "Press any key to continue" 
	SLEEP 
	omaQuit
    END IF 
    
    IF( TTF_Init() < 0 ) THEN
	SDL_Quit
	END 1
    END IF
    
    
    DIM FontSize AS INTEGER
    DIM FontFile AS STRING
    
    
    FontFile = "data/vera.ttf"
    FontSize = 11
    omaFont = TTF_OpenFont( FONTFILE, FONTSIZE )
    
    IF( omaFont = 0 ) THEN
	omaDebugEvent "Font failed to load"
	SLEEP
	omaQuit
    END IF
    
    omaFramesPerSecond = 30 
    omaFramePeriod = (1! / omaFramesPerSecond) * 1000! 
    
END SUB 

FUNCTION omaMultiKey (x as integer) as integer
    STATIC keys(0 TO 1024) AS SHORT 
    DIM event AS SDL_Event 
    DIM num as integer
    DIM a as integer
    DIM b as integer
    
    
    'HIGHEST = 319 LOWEST = 0 
    'So, keyflag protocol: 
    '0-320 will be keyflags 
    '320-330 will be mouseFlags 
    '350-370 will be Joystick 1 
    '380-400 will be Joystick 2 
    
    WHILE( SDL_PollEvent ( @event ) )                    
	SELECT CASE event.type 
	CASE SDL_KeyDOWN: 
	    keys(event.key.keysym.sym) = -1 
	CASE SDL_KeyUP: 
	    keys(event.key.keysym.sym) = 0 
	CASE SDL_MOUSEMOTION: 
	    '326,327,328,329 are mouse movement events 
	    keys(326) = (event.motion.xrel < 0)                
	    keys(327) = (event.motion.xrel > 0)                  
	    keys(328) = (event.motion.yrel < 0) 
	    keys(329) = (event.motion.yrel > 0) 
	CASE SDL_MOUSEBUTTONDown: 
	    'SDL mouse buttons 321->325 
	    num = 320+event.button.button 
	    keys(num) = -1
	    
	CASE SDL_MOUSEBUTTONUP: 
	    num = 320+event.button.button 
	    keys(num) = 0 
	    
	CASE SDL_JOYAXISMOTION 
	    'axis events on 
	    '400 + (Joysticknumber * 100) + (axis * 2) 
	    'There won't be more than 5 axises on a joystick, so x00-x20 are 
	    'reserved... 
	    num = 400 + (event.jaxis.which * 100) + (event.jaxis.axis * 2) 
	    keys(num) = (event.jaxis.value < 0)                
	    keys(num + 1) = (event.jaxis.value > 0) 
	    
	CASE SDL_JoyBallMotion 
	    
	    'jball 
	    'x20-x40 are Ball events. 20 is room for 5 balls. 
	    'There's something @ up if 5 balls isn't enough for you. 
	    'num=420 + (event.jaxis.which * 100) + (which * 4) 
	    num = 420 + (event.jball.which * 100) + (event.jaxis.axis * 2) 
	    keys(num) = (event.jball.xrel < 0)                
	    keys(num+1) = (event.jball.xrel > 0)                  
	    keys(num+2) = (event.jball.yrel < 0) 
	    keys(num+3) = (event.jball.yrel > 0) 
	    
	CASE SDL_JoyHatMotion 
	    'jhat 
	    'x40-x80 are Hat events.40 is room for 5 hats. 
	    'If you need 5 hats, I suggest surgery. 
	    num = 440 + (event.jhat.which * 100) + (event.jhat.hat * 8) 
	    FOR a = 1 TO 8 
		keys(num + a) = 0 
	    NEXT a 
	    keys(num + event.jhat.value) = -1 
	    
	CASE SDL_JoyButtonDown 
	    
	    'button events are x80-x99 
	    num = 40 + (event.jbutton.which * 100) + (event.jbutton.button) 
	    keys(num) = -1 
	CASE SDL_JoyButtonUp 
	    
	    num = 40 + (event.jbutton.which * 100) + (event.jbutton.button) 
	    keys(num) = 0 
	    
	END SELECT 
	
    WEND 
    omaMultiKey = keys(x) 
    
END FUNCTION 

FUNCTION omaGetSingleEvent AS INTEGER 
    
    DIM event AS SDL_Event 
    DIM num as integer
    DIM a as integer
    dim b as integer 
    'This function will return 0 if no event is in the queue, otherwise it'll return 
    'the keyflag protocol number. 
    
    'So, keyflag protocol: 
    '0-320 will be keyflags 
    '321-329 will be mouseFlags 
    '400-500 will be Joystick 1 
    '500-600 will be Joystick 2 
    '600-700 will be Joystick 3 
    '700-800 will be Joystick 4 
    '900-1000 will be Joystick 5 
    WHILE( SDL_PollEvent ( @event ) )                    
	SELECT CASE event.type 
	CASE SDL_KeyDOWN: 
	    omagetSingleEvent = event.key.keysym.sym 
	    EXIT FUNCTION 
	    
	CASE SDL_MOUSEMOTION: 
	    '326,327,328,329 are mouse movement events 
	    IF (event.motion.xrel < 0) THEN omagetSingleEvent = 326:EXIT FUNCTION 
	    IF (event.motion.xrel > 0) THEN omagetSingleEvent = 327:EXIT FUNCTION 
	    IF (event.motion.yrel < 0) THEN omagetSingleEvent = 328:EXIT FUNCTION 
	    IF (event.motion.yrel > 0) THEN omagetSingleEvent = 329:EXIT FUNCTION 
	    
	    
	CASE SDL_MOUSEBUTTONDown: 
	    'SDL mouse buttons 321->325 
	    omagetSingleEvent = 320+event.button.button 
	    EXIT FUNCTION 
	    
	CASE SDL_JOYAXISMOTION 
	    num = 400 + (event.jaxis.which * 100) + (event.jaxis.axis * 2) 
	    omagetSingleEvent = num 
	    EXIT FUNCTION 
	CASE SDL_JoyBallMotion 
	    
	    'jball 
	    'x20-x40 are Ball events. 20 is room for 5 balls. 
	    'There's something @ up if 5 balls isn't enough for you. 
	    'num=420 + (event.jaxis.which * 100) + (which * 4) 
	    num = 420 + (event.jball.which * 100) + (event.jaxis.axis * 2) 
	    IF (event.jball.xrel < 0) THEN omagetSingleEvent = num                
	    IF (event.jball.xrel > 0) THEN omagetSingleEvent = num + 1 
	    IF (event.jball.yrel < 0) THEN omagetSingleEvent = num + 2 
	    IF (event.jball.yrel > 0) THEN omagetSingleEvent = num + 3 
	    
	    EXIT FUNCTION 
	    
	CASE SDL_JoyHatMotion 
	    'jhat 
	    'x40-x80 are Hat events.40 is room for 5 hats. 
	    'If you need 5 hats, I suggest surgery. 
	    num = 440 + (event.jhat.which * 100) + (event.jhat.hat * 8) 
	    
	    omagetSingleEvent = num + event.jhat.value 
	    EXIT FUNCTION            
	CASE SDL_JoyButtonDown 
	    
	    'button events are x80-x99 
	    num = 40 + (event.jbutton.which * 100) + (event.jbutton.button) 
	    omagetSingleEvent = num 
	    EXIT FUNCTION 
	END SELECT 
	
    WEND 
    omaGetSingleEvent = 0 
    EXIT FUNCTION 
    
END FUNCTION 

SUB omaDebugEvent (Text as string) 
    IF omaDebugEvents THEN PRINT text 
END SUB 

SUB omaDebugEventsActive(status AS INTEGER) 
    omaDebugEvents = status 
END SUB 

SUB omaGetScreenDimensions (BYREF w AS INTEGER,BYREF h AS INTEGER) 
    w = omaScreenSurface->w 
    h = omaScreenSurface->h 
END SUB 
Sub omaQFAKBlitCulledImage(xc as integer, yc as integer, w as integer, h as integer, x as integer, y as integer, number as integer, image as SDL_Surface ptr)
    DIM Rectangle AS SDL_Rect 
    
    DIM Rectangle2 AS SDL_Rect 
    dim Rectangle3 as SDL_Rect
    rectangle3.x = xc
    Rectangle3.y = yc
    Rectangle3.w = W
    Rectangle3.h = H
    
    SDL_SetClipRect omaScreenSurface, @Rectangle3
    
    
    Rectangle.x = (number * 40) 
    Rectangle.y = 0 
    rectangle.w = 40
    rectangle.h = image->h 
    
    Rectangle2.x = x + xc
    Rectangle2.y = y + yc
    
    
    IF Rectangle2.x > omaScreenSurface->w OR rectangle2.y > omaScreenSurface->h THEN 
    ELSE
	SDL_BlitSurface image, @rectangle, omaScreenSurface, @rectangle2 
    END IF 
    
    SDL_SetClipRect omaScreenSurface, 0
    
end sub
SUB omaqfakBlitImage(x AS INTEGER,y AS INTEGER,number AS INTEGER, image AS sdl_surface PTR)  
    IF image = 0 THEN
	omaDebugEvent "That image doesn't exist!"
    END IF 
    DIM Rectangle AS SDL_Rect 
    
    DIM Rectangle2 AS SDL_Rect 
    
    
    Rectangle.x = (number * 40) '- 1
    Rectangle.y = 0 
    rectangle.w = 40
    rectangle.h = image->h 
    Rectangle2.x = x 
    Rectangle2.y = y 
    
    SDL_BlitSurface image, @rectangle, omaScreenSurface, @rectangle2 
    
END SUB 

SUB omadrawtext( BYVAL video AS SDL_Surface PTR, _
    BYVAL x AS INTEGER, BYVAL y AS INTEGER, BYVAL clr AS SDL_Color, _
    text AS STRING, BYVAL font AS TTF_Font ptr) STATIC
    
    DIM surface AS SDL_Surface PTR
    DIM rgbcolor AS SDL_Color
    
    surface = TTF_RenderText_Blended( font, text, clr )
    omaBlitImage x, y, surface
    SDL_FreeSurface( surface )
    
END SUB

SUB omaPrint(BYVAL x AS INTEGER, BYVAL y AS INTEGER, BYVAL clr AS SDL_Color, _
    text AS STRING) STATIC
    
    omaDrawText omaScreenSurface, x, y, clr, text, omaFont
    
END SUB
sub omaPaletteOverlay
    
    
    end sub
